@@ -30,11 +30,11 @@ from django.contrib.auth import login as auth_login, logout as auth_logout
 
 from django.contrib.auth import REDIRECT_FIELD_NAME, authenticate
 from django.contrib.auth.decorators import login_required
-from django.contrib.auth.forms import PasswordResetForm, SetPasswordForm
 from django.contrib.auth.tokens import default_token_generator
 from django.contrib.sites.models import get_current_site
 from django.http import HttpResponseRedirect
 from django.shortcuts import redirect
+from django.template.response import TemplateResponse
 from django.utils.decorators import method_decorator
 from django.utils.encoding import force_bytes
 from django.utils.http import urlsafe_base64_decode, urlsafe_base64_encode
@@ -50,7 +50,8 @@ from signup.auth import validate_redirect
 from signup.backends import get_email_backend
 from signup.decorators import check_user_active, _send_verification_email
 from signup.compat import User
-from signup.forms import NameEmailForm, PasswordChangeForm, UserForm
+from signup.forms import (NameEmailForm, PasswordChangeForm, PasswordResetForm,
+    UserForm)
 from signup.backends.auth import UsernameOrEmailAuthenticationForm
 from signup import signals
 from signup import settings
@@ -127,8 +128,18 @@ class PasswordResetConfirmBaseView(RedirectFormMixin, ProcessFormView):
     """
     Clicked on the link sent in the reset e-mail.
     """
+    form_class = PasswordChangeForm
     token_generator = default_token_generator
 
+    def get_context_data(self, **kwargs):
+        context = super(PasswordResetConfirmBaseView,
+                        self).get_context_data(**kwargs)
+        user = self.object
+        if user is not None and self.token_generator.check_token(
+                                    user, self.kwargs.get('token')):
+            context.update({'validlink': True})
+        return context
+
     def post(self, request, *args, **kwargs):
         """
         Handles POST requests, instantiating a form instance with the passed
@@ -136,15 +147,11 @@ class PasswordResetConfirmBaseView(RedirectFormMixin, ProcessFormView):
         """
         form_class = self.get_form_class()
         form = self.get_form(form_class)
-        try:
-            uid = urlsafe_base64_decode(self.kwargs.get('uidb64'))
-            user = User.objects.get(pk=uid)
-        except (TypeError, ValueError, OverflowError, User.DoesNotExist):
-            user = None
-
+        user = self.object
         if user is not None and self.token_generator.check_token(
                                     user, self.kwargs.get('token')):
             if form.is_valid():
+                form.save()
                 return self.form_valid(form)
         return self.form_invalid(form)
 
@@ -152,6 +159,19 @@ class PasswordResetConfirmBaseView(RedirectFormMixin, ProcessFormView):
         messages.info(self.request, "Your password has been reset sucessfully.")
         return super(PasswordResetConfirmBaseView, self).get_success_url()
 
+    def get_form_kwargs(self):
+        """
+        Returns the keyword arguments for instantiating the form.
+        """
+        kwargs = super(PasswordResetConfirmBaseView, self).get_form_kwargs()
+        try:
+            uid = urlsafe_base64_decode(self.kwargs.get('uidb64'))
+            self.object = User.objects.get(pk=uid)
+        except (TypeError, ValueError, OverflowError, User.DoesNotExist):
+            self.object = None
+        kwargs.update({'instance': self.object})
+        return kwargs
+
 
 class SignupBaseView(RedirectFormMixin, ProcessFormView):
     """
@@ -330,8 +350,8 @@ class RegistrationPasswordConfirmBaseView(RedirectFormMixin, ProcessFormView):
     form for entering a new password. We can activate the account for real
     once we know the email is valid and a password has been set.
     """
+    form_class = PasswordChangeForm
     token_generator = default_token_generator
-    form_class = SetPasswordForm
 
     @method_decorator(sensitive_post_parameters)
     @method_decorator(never_cache)
@@ -375,29 +395,73 @@ class RegistrationPasswordConfirmBaseView(RedirectFormMixin, ProcessFormView):
 
         return super(RegistrationPasswordConfirmBaseView, self).form_valid(form)
 
+    def get_context_data(self, **kwargs):
+        context = super(RegistrationPasswordConfirmBaseView,
+                        self).get_context_data(**kwargs)
+        user = self.object
+        if user is not None and self.token_generator.check_token(
+                                    user, self.kwargs.get('token')):
+            context.update({'validlink': True})
+        return context
 
-class PasswordResetView(TemplateResponseMixin, PasswordResetBaseView):
+    def get_form_kwargs(self):
+        """
+        Returns the keyword arguments for instantiating the form.
+        """
+        kwargs = super(RegistrationPasswordConfirmBaseView,
+                       self).get_form_kwargs()
+        try:
+            uid = urlsafe_base64_decode(self.kwargs.get('uidb64'))
+            self.object = User.objects.get(pk=uid)
+        except (TypeError, ValueError, OverflowError, User.DoesNotExist):
+            self.object = None
+        kwargs.update({'instance': self.object})
+        return kwargs
+
+
+class AuthTemplateResponseMixin(TemplateResponseMixin):
+    """
+    Returns a *disabled* page regardless when DISABLED_AUTHENTICATION
+    is True.
+    """
+
+    def dispatch(self, request, *args, **kwargs):
+        if request.method.lower() in self.http_method_names:
+            if settings.DISABLED_AUTHENTICATION:
+                context = {}
+                response_kwargs = {}
+                response_kwargs.setdefault('content_type', self.content_type)
+                #pylint:disable=star-args
+                return TemplateResponse(
+                    request=request, template='accounts/disabled.html',
+                    context=context, **response_kwargs)
+        return super(AuthTemplateResponseMixin, self).dispatch(
+            request, *args, **kwargs)
+
+
+class ActivationView(AuthTemplateResponseMixin, ActivationBaseView):
+
+    template_name = 'accounts/activate.html'
+
+
+class PasswordResetView(AuthTemplateResponseMixin, PasswordResetBaseView):
 
     template_name = 'accounts/recover.html'
 
 
-class PasswordResetConfirmView(TemplateResponseMixin,
+class PasswordResetConfirmView(AuthTemplateResponseMixin,
                                PasswordResetConfirmBaseView):
 
     template_name = 'accounts/reset.html'
 
 
-class SignupView(TemplateResponseMixin, SignupBaseView):
+class RegistrationPasswordConfirmView(AuthTemplateResponseMixin,
+                                      RegistrationPasswordConfirmBaseView):
 
-    template_name = 'accounts/register.html'
+    template_name = 'accounts/reset.html'
 
 
-class ActivationView(TemplateResponseMixin, ActivationBaseView):
-
-    template_name = 'accounts/activate.html'
-
-
-class SigninView(TemplateResponseMixin, SigninBaseView):
+class SigninView(AuthTemplateResponseMixin, SigninBaseView):
 
     template_name = 'accounts/login.html'
 
@@ -407,12 +471,11 @@ class SignoutView(TemplateResponseMixin, SignoutBaseView):
     template_name = 'accounts/logout.html'
 
 
-class RegistrationPasswordConfirmView(TemplateResponseMixin,
-                                      RegistrationPasswordConfirmBaseView):
+class SignupView(AuthTemplateResponseMixin, SignupBaseView):
 
-    template_name = 'accounts/reset.html'
+    template_name = 'accounts/register.html'
 
 
 @login_required
 def redirect_to_user_profile(request):
-    return redirect(request.user.get_absolute_url())
+    return redirect(reverse('users_profile', args=(request.user,)))
